@@ -5,7 +5,7 @@
   Copyright (C) 2006-2007 by Geoffrey R. Hutchison
 
   This file is part of the Avogadro molecular editor project.
-  For more information, see <http://avogadro.sourceforge.net/>
+  For more information, see <http://avogadro.openmolecules.net/>
 
   Some code is based on Open Babel
   For more information, see <http://openbabel.sourceforge.net/>
@@ -24,6 +24,8 @@
 #include <avogadro/primitive.h>
 #include <avogadro/color.h>
 #include <avogadro/glwidget.h>
+#include <avogadro/atom.h>
+#include <avogadro/primitivelist.h>
 
 #include <QProgressDialog>
 #include <QWriteLocker>
@@ -50,7 +52,8 @@ namespace Avogadro
       SeparatorIndex
     };
 
-  ForceFieldExtension::ForceFieldExtension( QObject *parent ) : Extension( parent ), m_molecule(0)
+  ForceFieldExtension::ForceFieldExtension( QObject *parent ) :
+    Extension( parent ), m_molecule(0)
   {
     QAction *action;
     // If you change this, see forcefielddialog.cpp, where we need to set the popup menu
@@ -63,8 +66,9 @@ namespace Avogadro
     if ( m_forceField ) { // make sure we can actually find and run it!
 
       action = new QAction( this );
-      action->setText( tr("Optimize Geometry" ));
+      action->setText( tr("&Optimize Geometry" ));
       action->setData(OptimizeGeometryIndex);
+      action->setShortcut( tr("Ctrl+Alt+O", "Shortcut key for optimize geometry") );
       m_actions.append( action );
 
       action = new QAction( this );
@@ -92,11 +96,10 @@ namespace Avogadro
       action->setData(SeparatorIndex);
       m_actions.append( action );
 
-      // Disable the constraints menu for now...
       action = new QAction( this );
       action->setText( tr("Constraints..." ));
       action->setData(ConstraintsIndex);
-      m_actions.append( action ); 
+      m_actions.append( action );
 
       action = new QAction( this );
       action->setText( tr("Ignore Selection" ));
@@ -107,6 +110,12 @@ namespace Avogadro
       action->setText( tr("Fix Selected Atoms" ));
       action->setData(FixAtomsIndex);
       m_actions.append( action );
+
+      // back in the main menu -- separate us from others
+      action = new QAction( this );
+      action->setSeparator(true);
+      action->setData(OptimizeGeometryIndex);
+      m_actions.append( action );
     }
     else
       qDebug() << "Unable to set up and use any force fields.";
@@ -116,22 +125,8 @@ namespace Avogadro
 
   ForceFieldExtension::~ForceFieldExtension()
   {
-    if (m_dialog) {
-      delete m_dialog;
-      m_dialog = 0;
-    }
-    if (m_conformerDialog) {
-      delete m_conformerDialog;
-      m_conformerDialog = 0;
-    }
-    if (m_constraintsDialog) {
-      delete m_constraintsDialog;
-      m_constraintsDialog = 0;
-    }
-    if (m_constraints) {
-      delete m_constraints;
-      m_constraints = 0;
-    }
+    delete m_constraints;
+    m_constraints = 0;
   }
 
   QList<QAction *> ForceFieldExtension::actions() const
@@ -146,7 +141,7 @@ namespace Avogadro
     if (i == OptimizeGeometryIndex)
       return QString();
 
-    return tr("&Extensions") + '>' + tr("&Molecular Mechanics");
+    return tr("E&xtensions") + '>' + tr("&Molecular Mechanics");
   }
 
   void ForceFieldExtension::setMolecule(Molecule *molecule)
@@ -161,7 +156,7 @@ namespace Avogadro
     ostringstream buff;
 
     if (!m_dialog)
-      m_dialog = new ForceFieldDialog;
+      m_dialog = new ForceFieldDialog(static_cast<QWidget*>(parent()));
     if (!m_constraints)
       m_constraints = new ConstraintsModel;
 
@@ -171,6 +166,7 @@ namespace Avogadro
     int i = action->data().toInt();
     double energy = 0.0;
     QString msg;
+    OBMol mol;
     switch ( i ) {
     case SetupForceFieldIndex: // setup force field
       m_dialog->show();
@@ -181,14 +177,16 @@ namespace Avogadro
 
       m_forceField->SetLogLevel( OBFF_LOGLVL_HIGH );
 
-      if ( !m_forceField->Setup( *m_molecule, m_constraints->constraints() ) ) {
+      mol = m_molecule->OBMol();
+      if ( !m_forceField->Setup( mol, m_constraints->constraints() ) ) {
         QMessageBox::warning( widget, tr( "Avogadro" ),
           tr( "Cannot set up the force field for this molecule." ));
         break;
       }
 
       energy = m_forceField->Energy();
-      msg = QString( tr( "Energy = %1 %2" ))
+      m_molecule->setEnergy(energy);
+      msg = QString( tr( "Energy = %L1 %2" ))
                      .arg(energy).arg(m_forceField->GetUnit().c_str());
       QMessageBox::information( widget, tr( "Avogadro" ), msg);
 
@@ -200,16 +198,17 @@ namespace Avogadro
 
       m_forceField->SetLogLevel( OBFF_LOGLVL_LOW );
 
-      if ( !m_forceField->Setup( *m_molecule, m_constraints->constraints() ) ) {
+      mol = m_molecule->OBMol();
+      if ( !m_forceField->Setup( mol, m_constraints->constraints() ) ) {
         QMessageBox::warning( widget, tr( "Avogadro" ),
           tr( "Cannot set up the force field for this molecule." ));
         break;
       }
 
       if (!m_conformerDialog)
-        m_conformerDialog = new ConformerSearchDialog;
+        m_conformerDialog = new ConformerSearchDialog(static_cast<QWidget*>(parent()));
 
-      m_conformerDialog->setup(m_molecule, m_forceField, m_constraints, 
+      m_conformerDialog->setup(m_molecule, m_forceField, m_constraints,
                                0, m_dialog->nSteps(), m_dialog->algorithm(), m_dialog->convergence());
       m_conformerDialog->show();
       break;
@@ -219,14 +218,15 @@ namespace Avogadro
 
       m_forceField->SetLogLevel( OBFF_LOGLVL_LOW );
 
-      if ( !m_forceField->Setup( *m_molecule, m_constraints->constraints() ) ) {
+      mol = m_molecule->OBMol();
+      if ( !m_forceField->Setup( mol, m_constraints->constraints() ) ) {
         QMessageBox::warning( widget, tr( "Avogadro" ),
           tr( "Cannot set up the force field for this molecule." ));
         break;
       }
-      
-      undo = new ForceFieldCommand( m_molecule, m_forceField, m_constraints, 
-                                    0, m_dialog->nSteps(), m_dialog->algorithm(), 
+
+      undo = new ForceFieldCommand( m_molecule, m_forceField, m_constraints,
+                                    0, m_dialog->nSteps(), m_dialog->algorithm(),
                                     m_dialog->convergence(), 0 );
 
       connect(undo, SIGNAL(message(QString)), this, SIGNAL(message(QString)));
@@ -235,7 +235,7 @@ namespace Avogadro
       break;
     case ConstraintsIndex: // show constraints dialog
       if (!m_constraintsDialog) {
-        m_constraintsDialog = new ConstraintsDialog;
+        m_constraintsDialog = new ConstraintsDialog(static_cast<QWidget*>(parent()));
         m_constraintsDialog->setModel(m_constraints);
       }
 
@@ -248,7 +248,7 @@ namespace Avogadro
       foreach(Primitive *p, selectedAtoms)
       {
         Atom *atom = static_cast<Atom *>(p);
-        m_constraints->addIgnore(atom->GetIdx());
+        m_constraints->addIgnore(atom->index() + 1);
       }
 
       m_forceField->SetConstraints(m_constraints->constraints());
@@ -261,7 +261,7 @@ namespace Avogadro
       foreach(Primitive *p, selectedAtoms)
       {
         Atom *atom = static_cast<Atom *>(p);
-        m_constraints->addAtomConstraint(atom->GetIdx());
+        m_constraints->addAtomConstraint(atom->index() + 1);
       }
 
       m_forceField->SetConstraints(m_constraints->constraints());
@@ -304,9 +304,37 @@ namespace Avogadro
     m_numConformers = numConformers;
   }
 
+  void ForceFieldThread::copyConformers()
+  {
+    OBMol obmol = m_molecule->OBMol();
+    // copy conformers to obmol
+    m_forceField->GetConformers( obmol );
+    // set the current conformer
+    obmol.SetConformer(m_cycles);
+    // copy the coordinates
+    double *coordPtr = obmol.GetCoordinates();
+    std::vector<Eigen::Vector3d> conformer;
+    foreach (Atom *atom, m_molecule->atoms()) {
+      while (conformer.size() < atom->id())
+        conformer.push_back(Eigen::Vector3d(0.0, 0.0, 0.0));
+      conformer.push_back(Eigen::Vector3d(coordPtr));
+      coordPtr += 3;
+    }
+
+    // add the conformer to m_molecule
+    m_molecule->addConformer(conformer, m_cycles);
+    // set it to the current conformer
+    m_molecule->setConformer(m_cycles);
+
+    // copy the conformer energies
+    if (obmol.HasData(OBGenericDataType::ConformerData)) {
+      OBConformerData *cd = (OBConformerData*) obmol.GetData(OBGenericDataType::ConformerData);
+      m_molecule->setEnergies(cd->GetEnergies());
+    }
+  }
+
   void ForceFieldThread::run()
   {
-    QWriteLocker locker( m_molecule->lock() );
     m_stop = false;
     m_cycles = 0;
 
@@ -316,7 +344,8 @@ namespace Avogadro
     m_forceField->SetLogFile( &buff );
     m_forceField->SetLogLevel( OBFF_LOGLVL_LOW );
 
-    if ( !m_forceField->Setup( *m_molecule, m_constraints->constraints() ) ) {
+    OBMol mol = m_molecule->OBMol();
+    if ( !m_forceField->Setup( mol, m_constraints->constraints() ) ) {
       qWarning() << "ForceFieldCommand: Could not set up force field on " << m_molecule;
       return;
     }
@@ -326,7 +355,26 @@ namespace Avogadro
         m_forceField->SteepestDescentInitialize( m_nSteps, pow( 10.0, -m_convergence )); // initialize sd
 
         while ( m_forceField->SteepestDescentTakeNSteps( 5 ) ) { // take 5 steps until convergence or m_nSteps taken
-          m_forceField->UpdateCoordinates( *m_molecule );
+          m_forceField->UpdateCoordinates( mol );
+          assert( mol.NumAtoms() == m_molecule->numAtoms() );
+          double *coordPtr = mol.GetCoordinates();
+          // forces
+          if (mol.HasData(OBGenericDataType::ConformerData)) {
+            OBConformerData *cd = (OBConformerData*) mol.GetData(OBGenericDataType::ConformerData);
+            const vector<vector<vector3> > &allForces = cd->GetForces();
+            if (allForces.size()) {
+              const vector<vector3> &forces = allForces[0];
+              if (forces.size() == mol.NumAtoms()) {
+                foreach(Atom* atom, m_molecule->atoms()) {
+                  atom->setForceVector(Eigen::Vector3d(forces[atom->index()].AsArray()));
+                }
+              }
+            }
+          }
+          foreach (Atom *atom, m_molecule->atoms()) {
+            atom->setPos(Eigen::Vector3d(coordPtr));
+            coordPtr += 3;
+          }
           m_molecule->update();
           m_cycles++;
           steps += 5;
@@ -341,8 +389,30 @@ namespace Avogadro
       } else if ( m_algorithm == 1 ) {
         m_forceField->ConjugateGradientsInitialize( m_nSteps, pow( 10.0, -m_convergence )); // initialize cg
 
+        OBMol mol;
         while ( m_forceField->ConjugateGradientsTakeNSteps( 5 ) ) { // take 5 steps until convergence or m_nSteps taken
-          m_forceField->UpdateCoordinates( *m_molecule );
+          mol = m_molecule->OBMol();
+          m_forceField->UpdateCoordinates( mol );
+          assert( mol.NumAtoms() == m_molecule->numAtoms() );
+          double *coordPtr = mol.GetCoordinates();
+          // forces
+          if (mol.HasData(OBGenericDataType::ConformerData)) {
+            OBConformerData *cd = (OBConformerData*) mol.GetData(OBGenericDataType::ConformerData);
+            const vector<vector<vector3> > &allForces = cd->GetForces();
+            if (allForces.size()) {
+              const vector<vector3> &forces = allForces[0];
+              if (forces.size() == mol.NumAtoms()) {
+                foreach(Atom* atom, m_molecule->atoms()) {
+                  atom->setForceVector(Eigen::Vector3d(forces[atom->index()].AsArray()));
+                }
+              }
+            }
+          }
+ 
+          foreach (Atom *atom, m_molecule->atoms()) {
+            atom->setPos(Eigen::Vector3d(coordPtr));
+            coordPtr += 3;
+          }
           m_molecule->update();
           m_cycles++;
           steps += 5;
@@ -358,7 +428,7 @@ namespace Avogadro
     } else if ( m_task == 1 ) {
       int n = m_forceField->SystematicRotorSearchInitialize(m_nSteps);
       while (m_forceField->SystematicRotorSearchNextConformer(m_nSteps)) {
-        m_forceField->GetConformers( *m_molecule );
+        copyConformers();
         m_molecule->update();
         m_cycles++;
         m_mutex.lock();
@@ -372,7 +442,7 @@ namespace Avogadro
     } else if ( m_task == 2 ) {
       m_forceField->RandomRotorSearchInitialize(m_numConformers, m_nSteps);
       while (m_forceField->RandomRotorSearchNextConformer(m_nSteps)) {
-        m_forceField->GetConformers( *m_molecule );
+        copyConformers();
         m_molecule->update();
         m_cycles++;
         m_mutex.lock();
@@ -385,11 +455,11 @@ namespace Avogadro
       }
     } else if ( m_task == 3 ) {
       m_forceField->WeightedRotorSearch(m_numConformers, m_nSteps);
-      m_forceField->GetConformers( *m_molecule );
       m_forceField->ConjugateGradients(250);
+      copyConformers();
     }
 
-    m_forceField->GetConformers( *m_molecule );
+    m_molecule->setEnergy(m_forceField->Energy());
     m_molecule->update();
 
     emit message( QObject::tr( buff.str().c_str() ) );
@@ -400,11 +470,12 @@ namespace Avogadro
   {
     QMutexLocker locker(&m_mutex);
     m_stop = true;
+    m_molecule->setEnergy(m_forceField->Energy());
   }
 
   ForceFieldCommand::ForceFieldCommand( Molecule *molecule, OpenBabel::OBForceField* forceField,
-                                        ConstraintsModel* constraints, 
-                                        int forceFieldID, int nSteps, int algorithm, 
+                                        ConstraintsModel* constraints,
+                                        int forceFieldID, int nSteps, int algorithm,
                                         int convergence, int task ) :
     m_nSteps( nSteps ),
     m_task( task ),
@@ -535,5 +606,5 @@ namespace Avogadro
 
 } // end namespace Avogadro
 
-#include "forcefieldextension.moc"
 Q_EXPORT_PLUGIN2(forcefieldextension, Avogadro::ForceFieldExtensionFactory)
+
