@@ -5,7 +5,7 @@
   Copyright (C) 2006 by Geoffrey R. Hutchison
 
   This file is part of the Avogadro molecular editor project.
-  For more information, see <http://avogadro.sourceforge.net/>
+  For more information, see <http://avogadro.openmolecules.net/>
 
   Some code is based on Open Babel
   For more information, see <http://openbabel.sourceforge.net/>
@@ -22,10 +22,14 @@
 
 #include "gamessextension.h"
 
-#include <avogadro/primitive.h>
+#include <avogadro/molecule.h>
+#include <avogadro/atom.h>
+#include <avogadro/bond.h>
 #include <avogadro/color.h>
 #include <avogadro/glwidget.h>
+#include <avogadro/primitivelist.h>
 
+#include <openbabel/mol.h>
 #include <openbabel/obiter.h>
 #include <openbabel/obconversion.h>
 
@@ -39,6 +43,7 @@
 #include <QSpacerItem>
 #include <QMessageBox>
 #include <QHeaderView>
+#include <QDebug>
 
 using namespace std;
 using namespace OpenBabel;
@@ -57,7 +62,7 @@ namespace Avogadro
       m_efpDialog( 0 ), m_qmDialog( 0 )
   {
     QAction *action = new QAction( this );
-    action->setText( tr("Input Deck Generator..." ));
+    action->setText( tr("Input Generator..." ));
     m_actions.append( action );
     action->setData( InputDeckAction );
 
@@ -89,14 +94,14 @@ namespace Avogadro
 
   QString GamessExtension::menuPath(QAction *) const
   {
-    return tr("&Extensions") + '>' + tr("&GAMESS");
+    return tr("E&xtensions") + '>' + tr("&GAMESS");
   }
 
   QDockWidget * GamessExtension::dockWidget()
   {
     if ( !m_gamessEfpDock ) {
       m_gamessEfpDock = new QDockWidget( tr( "GAMESS EFP Information" ) );
-      m_gamessEfpDock->setObjectName(tr("gamessEfpDock"));
+      m_gamessEfpDock->setObjectName("gamessEfpDock");
 
       QWidget *widget = new QWidget( m_gamessEfpDock );
       QVBoxLayout *layout = new QVBoxLayout();
@@ -188,7 +193,8 @@ namespace Avogadro
     return 0;
   }
 
-  GamessEfpMatchDialog *GamessExtension::matchesDialog( Molecule *molecule, GLWidget *widget, GamessEfpMatchDialog::Type type )
+  GamessEfpMatchDialog *GamessExtension::matchesDialog(Molecule *molecule,
+       GLWidget *widget, GamessEfpMatchDialog::Type type)
   {
     if ( !widget->selectedPrimitives().size() ) {
       QMessageBox::information( 0, tr( "No Atoms Selected" ),
@@ -204,53 +210,42 @@ namespace Avogadro
       return 0;
     }
 
+    m_molecule = molecule;
+
     PrimitiveList selectedPrimitives = widget->selectedPrimitives();
     Molecule selectedMolecule;
 
-    int numAtoms = 0;
-    std::map<OBAtom*, OBAtom*> AtomMap; // key is from old, value from new
-    foreach( Primitive *p, selectedPrimitives.subList(Primitive::AtomType) ) {
-      OBAtom *selected = static_cast<Atom*>( p );
-      selectedMolecule.InsertAtom( *selected );
-      AtomMap[selected] = selectedMolecule.GetAtom( selectedMolecule.NumAtoms() );
-      numAtoms++;
+    foreach(Primitive *p, selectedPrimitives.subList(Primitive::AtomType)) {
+      Atom *selected = static_cast<Atom*>(p);
+      Atom *atom = selectedMolecule.addAtom(selected->id());
+      *atom = *selected;
     }
 
     // use the atom map to map bonds
-    int numBonds = 0;
-    map<OBAtom*, OBAtom*>::iterator posBegin, posEnd;
-    FOR_BONDS_OF_MOL( b, molecule ) {
-      posBegin = AtomMap.find( b->GetBeginAtom() );
-      posEnd = AtomMap.find( b->GetEndAtom() );
+    foreach (Bond *b, m_molecule->bonds()) {
       // make sure both bonds are in the map (i.e. selected)
-      if ( posBegin != AtomMap.end() && posEnd != AtomMap.end() ) {
-        selectedMolecule.AddBond(( posBegin->second )->GetIdx(),
-                                 ( posEnd->second )->GetIdx(),
-                                 b->GetBO(), b->GetFlags() );
-        numBonds++;
+      if (selectedMolecule.atomById(b->beginAtomId()) &&
+          selectedMolecule.atomById(b->endAtomId())) {
+        Bond *bond = selectedMolecule.addBond(b->id());
+        bond->setAtoms(b->beginAtomId(), b->endAtomId(), b->order());
       }
     } // end looping over bonds
 
     // get the SMARTS pattern
-    string pattern = conv.WriteString( &selectedMolecule );
+    OpenBabel::OBMol obmol = selectedMolecule.OBMol();
+    string pattern = conv.WriteString( &obmol );
     pattern.erase( pattern.find_first_of( " \t\n\r" ) );
 
-//     QMessageBox::information( 0, tr( "" ), "Got Pattern : " + QString::fromStdString( pattern ) );
-
     OBSmartsPattern sp;
     sp.Init( pattern );
 
-    if ( sp.Match( *molecule ) ) {
+    if (sp.Match(obmol)) {
       // before we begin we need to see what has already been selected.
-
       QVector<Atom *> usedAtoms;
-
-      for(int parentNum = 0; parentNum < m_efpModel->rowCount(); parentNum++ )
-      {
+      for(int parentNum = 0; parentNum < m_efpModel->rowCount(); parentNum++ ) {
         QStandardItem *parentItem = m_efpModel->item(parentNum);
 
-        for(int childNum = 0; childNum < parentItem->rowCount(); childNum++ )
-        {
+        for(int childNum = 0; childNum < parentItem->rowCount(); childNum++ ) {
           QStandardItem *childItem = parentItem->child(childNum);
           QVector<Atom *> atoms = childItem->data().value<QVector<Atom *> >();
 
@@ -263,7 +258,8 @@ namespace Avogadro
 
       vector< vector<int> > maplist = sp.GetUMapList();
 
-      for ( vector< vector<int> >::iterator it1 = maplist.begin(); it1 != maplist.end(); it1++ ) {
+      for (vector< vector<int> >::iterator it1 = maplist.begin();
+           it1 != maplist.end(); it1++) {
 
         QVector<int> matches = QVector<int>::fromStdVector( *it1 );
 
@@ -272,40 +268,42 @@ namespace Avogadro
         bool first = true;
         bool selected = false;
         QVector<Atom *> atomMatches;
-        foreach( int i, matches ) {
-          Atom *atom = static_cast<Atom *>(molecule->GetAtom( i ));
+        foreach(int i, matches) {
+          Atom *atom = selectedMolecule.atom(i-1);
 
-          if(usedAtoms.contains(atom))
-          {
+          if(usedAtoms.contains(m_molecule->atomById(atom->id()))) {
             valid=false;
             break;
           }
 
-          atomMatches.append(atom);
+          atomMatches.append(m_molecule->atomById(atom->id()));
           // if this matches our original atom
-          if(!selected && selectedPrimitives.contains(atom)) {
+          if(!selected && selectedPrimitives.contains(m_molecule->atomById(atom->id()))) {
             selected = true;
           }
 
           if ( !first ) {
-            text.append( tr( ", " ) );
-          } else {
+            text.append( ", " );
+          }
+          else {
             first = false;
           }
 
           text.append( QString::number( i ) );
 
-          FOR_NBORS_OF_ATOM( a, atom ) {
+          foreach (unsigned long ai, atom->neighbors()) {
+            Atom *a = m_molecule->atom(ai);
             // all connected atoms must also be in our match
             // see if each neighbor is a hydrogen or another match
-            if ( a->IsHydrogen() ) {
+            if ( a->isHydrogen() ) {
               if ( !first ) {
-                text.append( tr( ", " ) );
+                text.append( ", " );
               }
-              int idx = a->GetIdx();
-              text.append( QString::number( idx ) );
-              atomMatches.append( static_cast<Atom *>(&*a) );
-            } else if ( !matches.contains( a->GetIdx() ) ) {
+              int idx = m_molecule->atomById(a->id())->index();
+              text.append(QString::number(idx));
+              atomMatches.append((m_molecule->atomById(a->id())));
+            }
+            else if (!matches.contains(selectedMolecule.atomById(a->id())->index()+1)) {
               valid = false;
               break;
             }
@@ -463,7 +461,8 @@ namespace Avogadro
 
   }
 
-  void GamessExtension::efpWidgetAccepted( const GamessEfpMatchDialog::Type &type, const QString &name, const QList<QVector<Atom *> > &groups )
+  void GamessExtension::efpWidgetAccepted( const GamessEfpMatchDialog::Type &type,
+                    const QString &name, const QList<QVector<Atom *> > &groups )
   {
     QString groupName = name;
     if ( !groupName.size() ) {
@@ -479,11 +478,11 @@ namespace Avogadro
       foreach( Atom *atom, group ) {
 
         if ( !first ) {
-          groupString.append( tr( ", " ) );
+          groupString.append( ", " );
         } else {
           first = false;
         }
-        groupString.append( QString::number( atom->GetIdx() ) );
+        groupString.append( QString::number( atom->index()+1 ) );
 
       }
 
@@ -520,7 +519,10 @@ namespace Avogadro
     if(newItems.size() && !rootItem)
     {
       rootItem = new QStandardItem(groupName);
-      QStandardItem *typeItem  = new QStandardItem(tr(" (") + tr( type ? "qm" : "efp" ) + tr(")") );
+      QString typeName = tr(" (");
+      typeName += type ? tr("qm") : tr("efp");
+      typeName += tr(")");
+      QStandardItem *typeItem  = new QStandardItem(typeName);
       QList<QStandardItem *> row;
       row.append(rootItem);
       row.append(typeItem);
@@ -591,11 +593,11 @@ namespace Avogadro
         foreach(Atom *atom, atoms)
         {
           if ( !first ) {
-            text.append( tr( ", " ) );
+            text.append( ", " );
           } else {
             first = false;
           }
-          text.append( QString::number( atom->GetIdx() ) );
+          text.append( QString::number( atom->index()+1 ) );
 
           if(atom == primitive)
           {
@@ -626,6 +628,5 @@ namespace Avogadro
 
 }
 
-#include "gamessextension.moc"
-
 Q_EXPORT_PLUGIN2(gamessextension, Avogadro::GamessExtensionFactory)
+
