@@ -5,7 +5,7 @@
   Copyright (C) 2006 by Geoffrey R. Hutchison
 
   This file is part of the Avogadro molecular editor project.
-  For more information, see <http://avogadro.sourceforge.net/>
+  For more information, see <http://avogadro.openmolecules.net/>
 
   Some code is based on Open Babel
   For more information, see <http://openbabel.sourceforge.net/>
@@ -21,19 +21,24 @@
  ***********************************************************************/
 
 #include "gamessinputdialog.h"
+#include "gamesshighlighter.h"
+
+#include <avogadro/molecule.h>
+#include <avogadro/atom.h>
+#include <avogadro/bond.h>
 
 #include <QPushButton>
 #include <QButtonGroup>
 #include <QDebug>
-
 #include <QFileDialog>
 #include <QFile>
-
 #include <QMessageBox>
 
 namespace Avogadro {
 
-  GamessInputDialog::GamessInputDialog( GamessInputData *inputData, QWidget *parent, Qt::WindowFlags f ) : QDialog( parent, f ), m_inputData(NULL), m_advancedChanged( false )
+  GamessInputDialog::GamessInputDialog( GamessInputData *inputData,
+    QWidget *parent, Qt::WindowFlags f ) : QDialog( parent, f ),
+    m_inputData(NULL), m_advancedChanged( false )
   {
     setInputData(inputData);
 
@@ -41,6 +46,8 @@ namespace Avogadro {
     ui.navigationTree->expandAll();
     ui.navigationTree->headerItem()->setHidden(true);
 
+    m_highlighter = new GamessHighlighter(ui.previewText->document());
+
     connectModes();
     connectBasic();
     connectAdvanced();
@@ -125,22 +132,6 @@ namespace Avogadro {
         this, SLOT( updatePreviewText() ) );
     connect( ui.basicOnRightCombo, SIGNAL( currentIndexChanged( int ) ),
         this, SLOT( basicChanged() ) );
-
-    // Time
-    connect( ui.basicTimeDouble, SIGNAL( valueChanged( double ) ),
-        this, SLOT( setBasicTime( double ) ) );
-    connect( ui.basicTimeDouble, SIGNAL( valueChanged( double ) ),
-        this, SLOT( updatePreviewText() ) );
-    connect( ui.basicTimeDouble, SIGNAL( valueChanged( double ) ),
-        this, SLOT( basicChanged() ) );
-
-    // Memory
-    connect( ui.basicMemoryDouble, SIGNAL( valueChanged( double ) ),
-        this, SLOT( setBasicMemory( double ) ) );
-    connect( ui.basicMemoryDouble, SIGNAL( valueChanged( double ) ),
-        this, SLOT( updatePreviewText() ) );
-    connect( ui.basicMemoryDouble, SIGNAL( valueChanged( double ) ),
-        this, SLOT( basicChanged() ) );
   }
 
   void GamessInputDialog::connectAdvanced()
@@ -223,10 +214,6 @@ namespace Avogadro {
     setBasicOnLeft(0);
     ui.basicOnRightCombo->setCurrentIndex( 2 );
     setBasicOnRight(2);
-    ui.basicTimeDouble->setValue( 10.0 );
-    setBasicTime(10.0);
-    ui.basicMemoryDouble->setValue( 50.0 );
-    setBasicMemory(50.0);
     updateBasicWidgets();
 
     blockChildrenSignals(ui.basicWidget, false);
@@ -382,9 +369,6 @@ namespace Avogadro {
     }
     setBasicOnRight( ui.basicOnRightCombo->currentIndex() );
 
-    setBasicTime( ui.basicTimeDouble->value() );
-    setBasicMemory( ui.basicMemoryDouble->value() );
-
     m_advancedChanged = false;
     ui.basicWidget->setEnabled( true );
   }
@@ -662,18 +646,6 @@ namespace Avogadro {
   {
     blockChildrenSignals( ui.systemWidget, true );
 
-    // timeLimitText
-    ui.systemTimeDouble->setValue( m_inputData->System->GetConvertedTime());
-
-    // timeLimitUnitCombo
-    ui.systemTimeCombo->setCurrentIndex( m_inputData->System->GetTimeUnits() - 1 );
-
-    // memoryText
-    ui.systemMemoryDouble->setValue( m_inputData->System->GetConvertedMem() );
-
-    // memoryUnitCombo
-    ui.systemMemoryCombo->setCurrentIndex( m_inputData->System->GetMemUnits() - 1 );
-
     // memDDI edit
     ui.systemDDIDouble->setValue( m_inputData->System->GetConvertedMemDDI() );
 
@@ -1126,34 +1098,6 @@ namespace Avogadro {
 
   void GamessInputDialog::connectSystem()
   {
-    connect( ui.systemTimeDouble, SIGNAL( valueChanged( double ) ),
-        this, SLOT( setSystemTime( double ) ) );
-    connect( ui.systemTimeDouble, SIGNAL( valueChanged( double ) ),
-        this, SLOT( updatePreviewText() ) );
-    connect( ui.systemTimeDouble, SIGNAL( valueChanged( double ) ),
-        this, SLOT( advancedChanged() ) );
-
-    connect( ui.systemTimeCombo, SIGNAL( currentIndexChanged( int ) ),
-        this, SLOT( setSystemTimeUnits( int ) ) );
-    connect( ui.systemTimeCombo, SIGNAL( currentIndexChanged( int ) ),
-        this, SLOT( updatePreviewText() ) );
-    connect( ui.systemTimeCombo, SIGNAL( currentIndexChanged( int ) ),
-        this, SLOT( advancedChanged() ) );
-
-    connect( ui.systemMemoryDouble, SIGNAL( valueChanged( double ) ),
-        this, SLOT( setSystemMemory( double ) ) );
-    connect( ui.systemMemoryDouble, SIGNAL( valueChanged( double ) ),
-        this, SLOT( updatePreviewText() ) );
-    connect( ui.systemMemoryDouble, SIGNAL( valueChanged( double ) ),
-        this, SLOT( advancedChanged() ) );
-
-    connect( ui.systemMemoryCombo, SIGNAL( currentIndexChanged( int ) ),
-        this, SLOT( setSystemMemoryUnits( int ) ) );
-    connect( ui.systemMemoryCombo, SIGNAL( currentIndexChanged( int ) ),
-        this, SLOT( updatePreviewText() ) );
-    connect( ui.systemMemoryCombo, SIGNAL( currentIndexChanged( int ) ),
-        this, SLOT( advancedChanged() ) );
-
     connect( ui.systemDDIDouble, SIGNAL( valueChanged( double ) ),
         this, SLOT( setSystemDDI( double ) ) );
     connect( ui.systemDDIDouble, SIGNAL( valueChanged( double ) ),
@@ -1651,8 +1595,14 @@ namespace Avogadro {
 
   void GamessInputDialog::generateClicked()
   {
+    QFileInfo defaultFile(m_inputData->m_molecule->fileName());
+    QString defaultPath = defaultFile.canonicalPath();
+    if (defaultPath.isEmpty())
+      defaultPath = QDir::homePath();
+
+    QString defaultFileName = defaultPath + '/' + defaultFile.baseName() + ".inp";
     QString fileName = QFileDialog::getSaveFileName(this, tr("Export Input Deck"),
-        "", tr("GAMESS Input Deck (*.inp)"));
+        defaultFileName, tr("GAMESS Input Deck (*.inp)"));
 
     if(fileName == "") return;
 
@@ -1835,7 +1785,7 @@ namespace Avogadro {
     {
       case 0:
         // Make sure we can select this.
-        if ( (numElectrons + charge & 1) ) {
+        if ( (numElectrons + charge) & 1 ) {
           ui.basicOnLeftCombo->setCurrentIndex(1);
           scf = GAMESS_ROHF;
           mult = 2;
@@ -1886,18 +1836,6 @@ namespace Avogadro {
     m_inputData->Control->SetCharge( charge );
   }
 
-  void GamessInputDialog::setBasicTime( double val )
-  {
-    m_inputData->System->SetConvertedTime( val );
-  }
-
-  void GamessInputDialog::setBasicMemory( double val )
-  {
-    // this should be the default
-    // m_inputData->System->SetMemUnits(megaBytesUnit);
-    m_inputData->System->SetConvertedMem( val );
-  }
-
   // Basis Slots
   void GamessInputDialog::setBasisSet( int index )
   {
@@ -2086,30 +2024,6 @@ namespace Avogadro {
   }
 
 
-  // System Slots
-  void GamessInputDialog::setSystemTime( double val )
-  {
-    m_inputData->System->SetConvertedTime( val );
-  }
-
-  void GamessInputDialog::setSystemTimeUnits( int index )
-  {
-    m_inputData->System->SetTimeUnits(( TimeUnit )( index+1 ) );
-    ui.systemTimeDouble->setValue( m_inputData->System->GetConvertedTime() );
-
-  }
-
-  void GamessInputDialog::setSystemMemory( double val )
-  {
-    m_inputData->System->SetConvertedMem( val );
-  }
-
-  void GamessInputDialog::setSystemMemoryUnits( int index )
-  {
-    m_inputData->System->SetMemUnits(( MemoryUnit )( index+1 ) );
-    ui.systemMemoryDouble->setValue( m_inputData->System->GetConvertedMem() );
-  }
-
   void GamessInputDialog::setSystemDDI( double val )
   {
     m_inputData->System->SetConvertedMemDDI( val );
@@ -2408,4 +2322,3 @@ namespace Avogadro {
   }
 }
 
-#include "gamessinputdialog.moc"
