@@ -5,7 +5,7 @@
   Copyright (C) 2008 Michael Banck
 
   This file is part of the Avogadro molecular editor project.
-  For more information, see <http://avogadro.sourceforge.net/>
+  For more information, see <http://avogadro.openmolecules.net/>
 
   Avogadro is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -25,11 +25,19 @@
 
 #include "gaussianinputdialog.h"
 
+#include <avogadro/molecule.h>
+#include <avogadro/atom.h>
+
+#include <openbabel/mol.h>
+
 #include <QString>
 #include <QTextStream>
 #include <QFileDialog>
 #include <QMessageBox>
+#include <QShowEvent>
+#include <QSettings>
 #include <QDebug>
+#include <QProcess>
 
 using namespace OpenBabel;
 
@@ -39,7 +47,8 @@ namespace Avogadro
   GaussianInputDialog::GaussianInputDialog(QWidget *parent, Qt::WindowFlags f)
     : QDialog(parent, f), m_molecule(0), m_title("Title"), m_calculationType(OPT),
     m_theoryType(B3LYP), m_basisType(B631Gd), m_multiplicity(1), m_charge(0),
-    m_procs(1), m_output(""), m_chk(false), m_coords(0), m_dirty(false)
+    m_procs(1), m_output(""), m_chk(false), m_coordType(CARTESIAN),
+    m_dirty(false), m_warned(false)
   {
     ui.setupUi(this);
     // Connect the GUI elements to the correct slots
@@ -67,6 +76,8 @@ namespace Avogadro
         this, SLOT(previewEdited()));
     connect(ui.generateButton, SIGNAL(clicked()),
         this, SLOT(generateClicked()));
+    connect(ui.computeButton, SIGNAL(clicked()),
+        this, SLOT(computeClicked()));
     connect(ui.resetButton, SIGNAL(clicked()),
         this, SLOT(resetClicked()));
     connect(ui.moreButton, SIGNAL(clicked()),
@@ -82,21 +93,85 @@ namespace Avogadro
   {
   }
 
+  void GaussianInputDialog::writeSettings(QSettings &settings) const
+  {
+    settings.setValue("gaussianCalcType", ui.calculationCombo->currentIndex());
+    settings.setValue("gaussianProcs", ui.procSpin->value());
+    settings.setValue("gaussianTheory", ui.theoryCombo->currentIndex());
+    settings.setValue("gaussianBasis", ui.basisCombo->currentIndex());
+    settings.setValue("gaussianOutput", ui.outputCombo->currentIndex());
+    settings.setValue("gaussianChk", ui.checkpointCheck->isChecked());
+    settings.setValue("gaussianCoord", ui.coordCombo->currentIndex());
+  }
+
+  void GaussianInputDialog::readSettings(QSettings &settings)
+  {
+    setProcs(settings.value("gaussianProcs", 2).toInt());
+    ui.procSpin->setValue(settings.value("gaussianProcs", 1).toInt());
+    setCalculation(settings.value("gaussianCalcType", 1).toInt());
+    ui.calculationCombo->setCurrentIndex(settings.value("gaussianCalcType", 1).toInt());
+    setTheory(settings.value("gaussianTheory", 3).toInt());
+    ui.theoryCombo->setCurrentIndex(settings.value("gaussianTheory", 3).toInt());
+    setBasis(settings.value("gaussianBasis", 2).toInt());
+    ui.basisCombo->setCurrentIndex(settings.value("gaussianBasis", 2).toInt());
+    setOutput(settings.value("gaussianOutput", 0).toInt());
+    ui.outputCombo->setCurrentIndex(settings.value("gaussianOutput", 0).toInt());
+    setChk(settings.value("gaussianChk", false).toBool());
+    ui.checkpointCheck->setChecked(settings.value("gaussianChk", false).toBool());
+    setCoords(settings.value("gaussianCoord", 0).toInt());
+    ui.coordCombo->setCurrentIndex(settings.value("gaussianCoord", 0).toInt());
+  }
+
+  void GaussianInputDialog::showEvent(QShowEvent *)
+  {
+    updatePreviewText();
+
+    if (pathToG03().isEmpty())
+      ui.computeButton->hide();
+    else
+      ui.computeButton->show();
+  }
+
+  QString GaussianInputDialog::pathToG03() const
+  {
+    QString returnPath;
+    QStringList pathList;
+
+    QStringList environment = QProcess::systemEnvironment();
+    // This is a pain
+    // Each item in the list is a key-value pair
+    // so we match PATH
+    // and then we split out the value (the bit after the =)
+    // and split the PATH by ':' characters
+    foreach(const QString &key, environment) {
+      if (key.startsWith(QLatin1String("PATH")))
+        pathList = key.split('=').at(1).split(':');
+    }
+
+    // I don't know how this works for Windows -- probably need a different method
+    foreach(const QString &path, pathList) {
+      QFileInfo info(path + '/' + "g03");
+      if (info.exists() && info.isExecutable())
+        returnPath = info.canonicalPath();
+    }
+
+    return returnPath;
+  }
+
   void GaussianInputDialog::setMolecule(Molecule *molecule)
   {
     // Disconnect the old molecule first...
-    if (m_molecule)
-    {
+    if (m_molecule) {
       disconnect(m_molecule, 0, this, 0);
     }
 
     m_molecule = molecule;
-    // Update the preview text whenever primitives are changed
-    connect(m_molecule, SIGNAL(primitiveRemoved(Primitive *)),
+    // Update the preview text whenever atoms are changed
+    connect(m_molecule, SIGNAL(atomRemoved(Atom *)),
             this, SLOT(updatePreviewText()));
-    connect(m_molecule, SIGNAL(primitiveAdded(Primitive *)),
+    connect(m_molecule, SIGNAL(atomAdded(Atom *)),
             this, SLOT(updatePreviewText()));
-    connect(m_molecule, SIGNAL(primitiveUpdated(Primitive *)),
+    connect(m_molecule, SIGNAL(atomUpdated(Atom *)),
             this, SLOT(updatePreviewText()));
     // Add atom coordinates
     updatePreviewText();
@@ -104,35 +179,35 @@ namespace Avogadro
 
   void GaussianInputDialog::updatePreviewText()
   {
+    if (!isVisible())
+      return;
     // Generate the input deck and display it
-    static bool called = false;
-    if (m_dirty && !called) {
-      called = true;
+    if (m_dirty && !m_warned) {
+      m_warned = true;
       QMessageBox msgBox;
 
-      msgBox.setWindowTitle("Gaussian Input Deck Generator Warning");
-      msgBox.setText("Would you like to update the preview text, losing all "
-        "changes made in the Gaussian input deck preview pane?");
+      msgBox.setWindowTitle(tr("Gaussian Input Deck Generator Warning"));
+      msgBox.setText(tr("Would you like to update the preview text, losing all changes made in the Gaussian input deck preview pane?"));
       msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
 
       switch (msgBox.exec()) {
         case QMessageBox::Yes:
           // yes was clicked
-          m_dirty = false;
+          deckDirty(false);
+          ui.previewText->setText(generateInputDeck());
+          m_warned = false;
           break;
         case QMessageBox::No:
           // no was clicked
+          m_warned = false;
           break;
         default:
           // should never be reached
           break;
       }
     }
-    
-    if (!m_dirty)
+    else if (!m_dirty)
       ui.previewText->setText(generateInputDeck());
-
-    called = false;
   }
 
   void GaussianInputDialog::resetClicked()
@@ -147,17 +222,139 @@ namespace Avogadro
     ui.procSpin->setValue(1);
   }
 
-  void GaussianInputDialog::generateClicked()
+  QString GaussianInputDialog::saveInputFile()
   {
+    QFileInfo defaultFile(m_molecule->fileName());
+    QString defaultPath = defaultFile.canonicalPath();
+    if (defaultPath.isEmpty())
+      defaultPath = QDir::homePath();
+
+    QString defaultFileName = defaultPath + '/' + defaultFile.baseName() + ".com";
     QString fileName = QFileDialog::getSaveFileName(this, tr("Save Gaussian Input Deck"),
-                                "", tr("Gaussian Input Deck (*.com)"));
+                                defaultFileName, tr("Gaussian Input Deck (*.com)"));
+
     QFile file(fileName);
     // FIXME This really should pop up a warning if the file cannot be opened
     if (!file.open(QIODevice::WriteOnly | QIODevice::Text))
-      return;
+      return QString();
+
+    QString previewText = ui.previewText->toPlainText();
+    QString checkpointName = QFileInfo(fileName).baseName();
+    checkpointName.prepend("%Chk=");
+    checkpointName.append(".chk");
+
+    previewText.replace(QLatin1String("%Chk=checkpoint.chk"), checkpointName, Qt::CaseInsensitive);
 
     QTextStream out(&file);
-    out << ui.previewText->toPlainText();
+    out << previewText;
+
+    return fileName;
+  }
+
+  void GaussianInputDialog::generateClicked()
+  {
+    saveInputFile();
+  }
+
+  void GaussianInputDialog::computeClicked()
+  {
+    if (m_process != 0) {
+      QMessageBox::warning(this, tr("Gaussian Running."),
+                           tr("Gaussian is already running. Wait until the previous calculation is finished."));
+      return;
+    }
+
+    QString fileName = saveInputFile();
+    if (fileName.isEmpty())
+      return;
+
+    QFileInfo info(pathToG03() + '/' + "g03");
+    if (!info.exists() || !info.isExecutable()) {
+      QMessageBox::warning(this, tr("Gaussian Not Installed."),
+                           tr("The G03 executable, cannot be found."));
+      return;
+    }
+
+    m_process = new QProcess(this);
+    QFileInfo input(fileName);
+    m_process->setWorkingDirectory(input.absolutePath());
+
+    QStringList arguments;
+    arguments << fileName;
+    m_inputFile = fileName; // save for reading in output
+
+    m_process->start(pathToG03() + '/' + "g03", arguments);
+    if (!m_process->waitForStarted()) {
+      QMessageBox::warning(this, tr("G03 failed to start."),
+                           tr("G03 did not start. Perhaps it is not installed correctly."));
+    }
+    connect(m_process, SIGNAL(finished(int)), this, SLOT(finished(int)));
+    m_progress = new QProgressDialog(this);
+    m_progress->setRange(0,0); // indeterminate progress
+    m_progress->setLabelText(tr("Running Gaussian calculation..."));
+    m_progress->show();
+    connect(m_progress, SIGNAL(canceled()), this, SLOT(stopProcess()));
+  }
+
+  void GaussianInputDialog::stopProcess()
+  {
+    if (m_progress) {
+      m_progress->deleteLater();
+      m_progress = 0;
+    }
+
+    disconnect(m_process, 0, this, 0); // don't send a "finished" signal
+    m_process->close();
+    m_process->deleteLater();
+    m_process = 0;
+  }
+
+  void GaussianInputDialog::finished(int exitCode)
+  {
+    if (m_progress) {
+      m_progress->cancel();
+      m_progress->deleteLater();
+      m_progress = 0;
+    }
+
+    if (m_process) {
+      disconnect(m_process, 0, this, 0);
+      m_process->deleteLater();
+      m_process = 0;
+    } else {
+      return; // we probably cancelled
+    }
+
+    if (exitCode) {
+      QMessageBox::warning(this, tr("G03 Crashed."),
+                           tr("Gaussian did not run correctly. Perhaps it is not installed correctly."));
+     return;
+    }
+
+    if (!m_molecule)
+      return;
+
+    // we have a successful run.
+    // try to run formchk
+    QFileInfo inputFile(m_inputFile);
+    QString checkpointFileName = inputFile.canonicalPath() + '/' + inputFile.baseName() + ".chk";
+    QFileInfo checkpointFile(checkpointFileName);
+    if (checkpointFile.exists() && checkpointFile.isReadable()) {
+      // let's see if formchk exists
+      QString formchkFilePath = pathToG03() + '/' + "formchk";
+      QFileInfo formchkInfo(formchkFilePath);
+      if (formchkInfo.exists() && formchkInfo.isExecutable()) {
+        QStringList arguments;
+        arguments << checkpointFileName;
+        QProcess::execute(formchkFilePath, arguments); // if this fails, it's not a big deal
+      }
+    }
+    
+    // Now, read in the results and close the dialog
+    QString outputFile = inputFile.canonicalPath() + '/' + inputFile.baseName() + ".log";
+    emit readOutput(outputFile);
+
+    close();
   }
 
   void GaussianInputDialog::moreClicked()
@@ -175,6 +372,7 @@ namespace Avogadro
 
   void GaussianInputDialog::enableFormClicked()
   {
+    deckDirty(false);
     updatePreviewText();
   }
 
@@ -237,12 +435,12 @@ namespace Avogadro
       default:
         m_theoryType = RHF;
     }
-    
+
     if (m_theoryType == AM1 || m_theoryType == PM3)
       ui.basisCombo->setEnabled(false);
     else
       ui.basisCombo->setEnabled(true);
-    
+
     updatePreviewText();
   }
 
@@ -262,6 +460,9 @@ namespace Avogadro
       case 3:
         m_basisType = B631Gdp;
         break;
+      case 4:
+        m_basisType = LANL2DZ;
+        break;
       default:
         m_basisType = B631Gd;
     }
@@ -311,7 +512,20 @@ namespace Avogadro
 
   void GaussianInputDialog::setCoords(int n)
   {
-    m_coords = n;
+    switch (n)
+    {
+      case 0:
+        m_coordType = CARTESIAN;
+        break;
+      case 1:
+        m_coordType = ZMATRIX;
+        break;
+      case 2:
+        m_coordType = ZMATRIX_COMPACT;
+        break;
+      default:
+        m_coordType = CARTESIAN;
+    }
     updatePreviewText();
   }
 
@@ -323,19 +537,20 @@ namespace Avogadro
 
     // These directives are required before the job specification
     if (m_procs > 1)
-      mol << "%NProcShared=" << m_procs << "\n";
-    if (m_chk)
+      mol << "%NProcShared=" << m_procs << '\n';
+    if (m_chk) {
       mol << "%Chk=checkpoint.chk\n";
+    }
 
     // Now specify the job type etc
-    mol << "#p " << getTheoryType(m_theoryType);
+    mol << "#n " << getTheoryType(m_theoryType);
 
     // Not all theories have a basis set
     if (m_theoryType != AM1 && m_theoryType != PM3)
-      mol << "/" << getBasisType(m_basisType);
+      mol << '/' << getBasisType(m_basisType);
 
     // Now for the calculation type
-    mol << " " << getCalculationType(m_calculationType);
+    mol << ' ' << getCalculationType(m_calculationType);
 
     // Output parameters for some programs
     mol << m_output;
@@ -344,24 +559,81 @@ namespace Avogadro
     mol << "\n\n " << m_title << "\n\n";
 
     // Now for the charge and multiplicity
-    mol << m_charge << " " << m_multiplicity << "\n";
+    mol << m_charge << ' ' << m_multiplicity << '\n';
 
     // Now to output the actual molecular coordinates
     // Cartesian coordinates
-    if (m_molecule && m_coords == 0)
-    {
+    if (m_molecule && m_coordType == CARTESIAN) {
       QTextStream mol(&buffer);
-      FOR_ATOMS_OF_MOL(atom, m_molecule)
-      {
-        mol << qSetFieldWidth(3) << left << QString(etab.GetSymbol(atom->GetAtomicNum()))
+      QList<Atom *> atoms = m_molecule->atoms();
+      foreach (Atom *atom, atoms) {
+        mol << qSetFieldWidth(3) << left
+            << QString(OpenBabel::etab.GetSymbol(atom->atomicNumber()))
             << qSetFieldWidth(15) << qSetRealNumberPrecision(5) << forcepoint
-            << fixed << right << atom->GetX() << atom->GetY() << atom->GetZ()
-            << qSetFieldWidth(0) << "\n";
+            << fixed << right << atom->pos()->x() << atom->pos()->y()
+            << atom->pos()->z()
+            << qSetFieldWidth(0) << '\n';
       }
-      mol << "\n";
+      mol << '\n';
     }
     // Z-matrix
-    if (m_molecule && m_coords == 1)
+    else if (m_molecule && m_coordType == ZMATRIX) {
+      QTextStream mol(&buffer);
+      OpenBabel::OBAtom *a, *b, *c;
+      double r, w, t;
+
+      /* Taken from OpenBabel's gzmat file format converter */
+      std::vector<OpenBabel::OBInternalCoord*> vic;
+      vic.push_back((OpenBabel::OBInternalCoord*)NULL);
+      OpenBabel::OBMol obmol = m_molecule->OBMol();
+      FOR_ATOMS_OF_MOL(atom, &obmol)
+        vic.push_back(new OpenBabel::OBInternalCoord);
+      CartesianToInternal(vic, obmol);
+
+      foreach (Atom *atom, m_molecule->atoms()) {
+        a = vic[atom->index()+1]->_a;
+        b = vic[atom->index()+1]->_b;
+        c = vic[atom->index()+1]->_c;
+
+        mol << qSetFieldWidth(3) << left
+            << QString(OpenBabel::etab.GetSymbol(atom->atomicNumber()))
+            << qSetFieldWidth(0);
+        if (atom->index() > 0)
+          mol << ' ' << a->GetIdx() << " r" << atom->index();
+        if (atom->index() > 1)
+          mol << ' ' << b->GetIdx() << " a" << atom->index();
+        if (atom->index() > 2)
+          mol << ' ' << c->GetIdx() << " d" << atom->index();
+        mol << '\n';
+      }
+
+      mol << "Variables:" << endl;
+      foreach (Atom *atom, m_molecule->atoms()) {
+        r = vic[atom->index()+1]->_dst;
+        w = vic[atom->index()+1]->_ang;
+        if (w < 0.0)
+          w += 360.0;
+        t = vic[atom->index()+1]->_tor;
+        if (t < 0.0)
+          t += 360.0;
+        if (atom->index() > 0)
+          mol << "r" << atom->index() << qSetFieldWidth(15)
+              << qSetRealNumberPrecision(5) << forcepoint << fixed << right
+              << r << qSetFieldWidth(0) << '\n';
+        if (atom->index() > 1)
+          mol << "a" << atom->index() << qSetFieldWidth(15)
+              << qSetRealNumberPrecision(5) << forcepoint << fixed << right
+              << w << qSetFieldWidth(0) << '\n';
+        if (atom->index() > 2)
+          mol << "d" << atom->index() << qSetFieldWidth(15)
+              << qSetRealNumberPrecision(5) << forcepoint << fixed << right
+              << t << qSetFieldWidth(0) << '\n';
+      }
+      mol << '\n';
+      foreach(OpenBabel::OBInternalCoord *c, vic)
+        delete c;
+    }
+    else if (m_molecule && m_coordType == ZMATRIX_COMPACT)
     {
       QTextStream mol(&buffer);
       OBAtom *a, *b, *c;
@@ -370,30 +642,16 @@ namespace Avogadro
       /* Taken from OpenBabel's gzmat file format converter */
       std::vector<OBInternalCoord*> vic;
       vic.push_back((OBInternalCoord*)NULL);
-      FOR_ATOMS_OF_MOL(atom, m_molecule)
+      OpenBabel::OBMol obmol = m_molecule->OBMol();
+      FOR_ATOMS_OF_MOL(atom, &obmol)
         vic.push_back(new OBInternalCoord);
-      CartesianToInternal(vic, (OpenBabel::OBMol&)*m_molecule);
+      CartesianToInternal(vic, obmol);
 
-      FOR_ATOMS_OF_MOL(atom, m_molecule)
+      FOR_ATOMS_OF_MOL(atom, &obmol)
       {
         a = vic[atom->GetIdx()]->_a;
         b = vic[atom->GetIdx()]->_b;
         c = vic[atom->GetIdx()]->_c;
-
-        mol << qSetFieldWidth(3) << left << QString(etab.GetSymbol(atom->GetAtomicNum()))
-            << qSetFieldWidth(0);
-        if (atom->GetIdx() > 1)
-          mol << " " << a->GetIdx() << " r" << atom->GetIdx();
-        if (atom->GetIdx() > 2)
-          mol << " " << b->GetIdx() << " a" << atom->GetIdx();
-        if (atom->GetIdx() > 3)
-          mol << " " << c->GetIdx() << " d" << atom->GetIdx();
-        mol << "\n";
-      }
-
-      mol << "Variables:" << endl;
-      FOR_ATOMS_OF_MOL(atom, m_molecule)
-      {
         r = vic[atom->GetIdx()]->_dst;
         w = vic[atom->GetIdx()]->_ang;
         if (w < 0.0)
@@ -401,20 +659,23 @@ namespace Avogadro
         t = vic[atom->GetIdx()]->_tor;
         if (t < 0.0)
           t += 360.0;
+
+        mol << qSetFieldWidth(3) << left << QString(etab.GetSymbol(atom->GetAtomicNum()))
+            << qSetFieldWidth(6) << right;
         if (atom->GetIdx() > 1)
-          mol << "r" << atom->GetIdx() << qSetFieldWidth(15)
-              << qSetRealNumberPrecision(5) << forcepoint << fixed << right
-              << r << qSetFieldWidth(0) << "\n";
+          mol << a->GetIdx() << qSetFieldWidth(15)
+          << qSetRealNumberPrecision(5) << forcepoint << fixed << right << r;
         if (atom->GetIdx() > 2)
-          mol << "a" << atom->GetIdx() << qSetFieldWidth(15)
-              << qSetRealNumberPrecision(5) << forcepoint << fixed << right
-              << w << qSetFieldWidth(0) << "\n";
+          mol << qSetFieldWidth(6) << right << b->GetIdx() << qSetFieldWidth(15)
+          << qSetRealNumberPrecision(5) << forcepoint << fixed << right << w;
         if (atom->GetIdx() > 3)
-          mol << "d" << atom->GetIdx() << qSetFieldWidth(15)
-              << qSetRealNumberPrecision(5) << forcepoint << fixed << right
-              << t << qSetFieldWidth(0) << "\n";
+          mol << qSetFieldWidth(6) << right << c->GetIdx() << qSetFieldWidth(15)
+          << qSetRealNumberPrecision(5) << forcepoint << fixed << right << t;
+        mol << qSetFieldWidth(0) << '\n';
       }
-      mol << "\n";
+      mol << '\n';
+      foreach(OpenBabel::OBInternalCoord *c, vic)
+        delete c;
     }
 
     return buffer;
@@ -471,6 +732,8 @@ namespace Avogadro
         return "6-31G(d)";
       case B631Gdp:
         return "6-31G(d,p)";
+      case LANL2DZ:
+        return "LANL2DZ";
       default:
         return "6-31G(d)";
     }
@@ -493,4 +756,3 @@ namespace Avogadro
 
 }
 
-#include "gaussianinputdialog.moc"
