@@ -1,12 +1,12 @@
 /**********************************************************************
   GLWidget - general OpenGL display
 
-  Copyright (C) 2006,2007 Geoffrey R. Hutchison
+  Copyright (C) 2006-2009 Geoffrey R. Hutchison
   Copyright (C) 2006,2007 Donald Ephraim Curtis
-  Copyright (C) 2007,2008 Marcus D. Hanwell
+  Copyright (C) 2007-2009 Marcus D. Hanwell
 
   This file is part of the Avogadro molecular editor project.
-  For more information, see <http://avogadro.sourceforge.net/>
+  For more information, see <http://avogadro.openmolecules.net/>
 
   Avogadro is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -28,126 +28,37 @@
 #define GLWIDGET_H
 
 #include <avogadro/global.h>
-#include <avogadro/engine.h>
-#include <avogadro/tool.h>
-#include <avogadro/color.h>
-#include <avogadro/painter.h>
-#include <avogadro/camera.h>
+
+#ifdef ENABLE_GLSL
+  #include <GL/glew.h>
+#endif
+#include <QGLWidget>
+
+#include <avogadro/glhit.h>
 
 #include <QThread>
 
-#include <vector>
+#include <Eigen/Core>
 
 class QUndoStack;
 class QMouseEvent;
 class QGLContext;
+class QSettings;
 
 namespace Avogadro {
 
+  class Primitive;
+  class Atom;
+  class Bond;
+  class Molecule;
+  class Camera;
+  class Painter;
+  class Tool;
   class ToolGroup;
-
-  /**
-   * @class GLHit glwidget.h <avogadro/glwidget.h>
-   * @brief Class for wrapping hits from GL picking.
-   * @author Donald Ephraim Curtis
-   *
-   * Provides an easy to use class to contain OpenGL hits returned from the
-   * process of picking.  This class relies on the %Engine subclasses properly
-   * naming the objects that they are rendering.  For more information see the
-   * Engine documentation.
-   */
-  class GLHitPrivate;
-  class A_EXPORT GLHit
-  {
-    public:
-      /**
-       * Blank constructor.
-       */
-      GLHit();
-      /**
-       * Copy constructor.
-       */
-      GLHit(const GLHit &glHit);
-      /**
-       * Constructor.
-       * @param type The type of the OpenGL object that was picked which corresponds
-       * to the Primitive::Type for the object
-       * (ie. type==Primitive::AtomType means an Atom was picked).
-       * @param name The name of the OpenGL object that was picked corresponding
-       * to the primitive index
-       * (ie. name==1 means Atom 1).
-       * @param minZ minimum window Z value of the hit
-       * @param maxZ maximum window Z value of the hit
-       */
-      GLHit(GLuint type, GLuint name, GLuint minZ, GLuint maxZ);
-      /**
-       * Destructor.
-       */
-      ~GLHit();
-
-      /**
-       * Less than operator.
-       * @param other the other GLHit object to compare to.
-       * @return (this->minZ < other->minZ) ? @c true : @c false
-       */
-      bool operator<(const GLHit &other) const;
-
-      /**
-       * Equivalence operator.
-       * @param other the other GLHit object to test equivalence with.
-       * @return returns true if all elements are equivalent (type, name, minZ, maxZ).
-       */
-      bool operator==(const GLHit &other) const;
-
-      /**
-       * Copy operator.
-       * @param other the GLHit object to set this object equal to.
-       * @return  *this
-       */
-      GLHit &operator=(const GLHit &other);
-
-      /**
-       * @return type of the object that was picked.
-       */
-      GLuint type() const;
-
-      /**
-       * @return name of the object that was picked.
-       */
-      GLuint name() const;
-
-      /**
-       * @return the minimum z value of this hit corresponding
-       * to the z value of the drawn object closest to the camera.
-       */
-      GLuint minZ() const;
-
-      /**
-       * @return the maximum z value of this hit corresponding
-       * to the z value of the drawn object farthest from the camera.
-       */
-      GLuint maxZ() const;
-
-      /**
-       * @param type new object type.
-       */
-      void setType(GLuint type);
-      /**
-       * @param name new object name.
-       */
-      void setName(GLuint name);
-      /**
-       * @param minZ minimum z value to set for this object.
-       */
-      void setMinZ(GLuint minZ);
-      /**
-       * @param maxZ maximum z value to set for this object.
-       */
-      void setMaxZ(GLuint maxZ);
-
-    private:
-      GLHitPrivate * const d;
-  };
+  class Color;
+  class Engine;
+  class Painter;
+  class PrimitiveList;
 
   /**
    * @class GLWidget glwidget.h <avogadro/glwidget.h>
@@ -208,15 +119,24 @@ namespace Avogadro {
       ~GLWidget();
 
       /**
-       * @return true if the GLWidget is stable as determined
-       * by the tools and extensions.
+       * @param enabled True if quick render is desired when moving the view.
        */
-      bool isStable() const;
+      void setQuickRender(bool enabled);
 
       /**
-       * @param stable the new stable value.
+       * @return True if quick rendering is being used.
        */
-      void setStable(bool stable);
+      bool quickRender() const;
+
+      /**
+      * @param enabled True if we should render the unit cell axes
+      */
+      void setRenderUnitCellAxes(bool enabled);
+
+      /**
+       * @return True if unit cell axes are being rendered.
+       */
+      bool renderUnitCellAxes() const;
 
       /**
        * @return the width of the widget in pixels.
@@ -253,11 +173,6 @@ namespace Avogadro {
        */
       Tool* tool() const;
 
-      /**
-       * @return the ToolGroup manager.
-       */
-      ToolGroup toolManger() const;
-
       /**
        * @return the current background color of the rendering area.
        */
@@ -308,11 +223,6 @@ namespace Avogadro {
        */
       QList<Engine *> engines() const;
 
-      /**
-       * @return a list of the engine factories.
-       */
-      QList<EngineFactory *> engineFactories() const;
-
       /**
        * Get the hits for a region starting at (x, y) of size (w * h).
        */
@@ -350,7 +260,7 @@ namespace Avogadro {
       /**
        * @return the radius of the Molecule.
        */
-      const double & radius() const;
+      double radius() const;
       /**
        * @return the Atom farthest away from the camera.
        */
@@ -360,11 +270,22 @@ namespace Avogadro {
        * @param quality set the global quality of the widget.
        */
       void setQuality(int quality);
+
       /**
        * @return the global quality of the widget.
        */
       int quality() const;
 
+      /**
+       * @param level Set the global fog level of the widget.
+       */
+      void setFogLevel(int level);
+
+      /**
+       * @return The global fog level of the widget.
+       */
+      int fogLevel() const;
+
       /**
        * Set to render x, y, z axes as an overlay in the bottom left of the widget.
        */
@@ -404,11 +325,6 @@ namespace Avogadro {
         */
       Painter *painter() const;
 
-      /**
-       * @return list of primitives for this widget
-       */
-      PrimitiveList primitives() const;
-
       /** @name Selection Methods
        *  These methods are used to manipulate user-selected primitives.
        *  Each view tracks a list of selected objects (e.g., atoms)
@@ -430,6 +346,12 @@ namespace Avogadro {
        */
       void toggleSelected(PrimitiveList primitives);
 
+      /**
+       * Toggle the selection for the GLWidget, that is if the primitive is
+       * selected, deselect it and vice-versa.
+       */
+      void toggleSelected();
+
       /**
        * Change the selection status for the atoms in the supplied list.
        * All objects in the list will have the same selection status.
@@ -437,7 +359,7 @@ namespace Avogadro {
        * @param primitives the set of objects to update.
        * @param select whether to select or deselect the objects.
        */
-      void setSelected(PrimitiveList primitives, bool select = true);
+      void setSelected(PrimitiveList primitives, bool select = true); // do we pass by value intentionally
 
       /**
        * Deselect all objects.
@@ -448,6 +370,54 @@ namespace Avogadro {
        * @return true if the Primitive is selected.
        */
       bool isSelected(const Primitive *p) const;
+
+      /**
+       * Add a new named selection.
+       *
+       * @param name name of the new selection.
+       * @param primitives the primitives ids.
+       * @return true if the name is not taken
+       */
+      bool addNamedSelection(const QString &name, PrimitiveList &primitives);
+      /**
+       * Remove a named selection by name.
+       *
+       * @param name name of the selection to remove.
+       */
+      void removeNamedSelection(const QString &name);
+      /**
+       * Remove a named selection by index. Using the index is useful in Models.
+       *
+       * @param index index of the selection to remove.
+       */
+      void removeNamedSelection(int index);
+      /**
+       * Rename a named selection by index. Using the index is useful in Models.
+       *
+       * @param index index of the selection to rename.
+       */
+      void renameNamedSelection(int index, const QString &name);
+      /**
+       * Get the names of all named selections.
+       *
+       * @return a list with all the named selections.
+       */
+      QList<QString> namedSelections();
+      /**
+       * Get the primitives of a named selections by name.
+       *
+       * @param name name of the selection.
+       * @return the primitives for this named selection.
+       */
+      PrimitiveList namedSelectionPrimitives(const QString &name);
+      /**
+       * Get the primitives of a named selections by index.
+       * Using the index is useful in Models.
+       *
+       * @param index index of the selection.
+       * @return the primitives for this named selection.
+       */
+      PrimitiveList namedSelectionPrimitives(int index);
       /* end selection method grouping */
       /** @} */
 
@@ -463,22 +433,22 @@ namespace Avogadro {
       void setUnitCells(int a, int b, int c);
 
       /**
-        * Clear the unit cell data
-      */
+       * Clear the unit cell data.
+       */
       void clearUnitCell();
 
       /**
-       * @return the number of unit cells to display along the a axis.
+       * @return The number of unit cells to display along the a axis.
        */
       int aCells();
 
       /**
-       * @return the number of unit cells to display along the b axis.
+       * @return The number of unit cells to display along the b axis.
        */
       int bCells();
 
       /**
-       * @return the number of unit cells to display along the c axis.
+       * @return The number of unit cells to display along the c axis.
        */
       int cCells();
 
@@ -497,6 +467,11 @@ namespace Avogadro {
        */
       static void setCurrent(GLWidget *current);
 
+      /**
+       * Trigger a render for GL2PS.
+       */
+      void renderNow();
+
       /**
        * Write the settings of the GLWidget in order to save them to disk.
        */
@@ -509,6 +484,7 @@ namespace Avogadro {
 
 
     protected:
+      friend class GLGraphicsView;
       /**
        * Virtual function called by QGLWidget on initialization of
        * the GL area.
@@ -521,6 +497,17 @@ namespace Avogadro {
        */
       virtual void paintGL();
 
+      /**
+       * Virtual function called by GLGraphicsView before render() to set up the
+       * display correctly and leave it in an appropriate state after.
+       */
+      virtual void paintGL2();
+
+      /**
+       * Virtual function called whn the GL area is resized
+       */
+      virtual void resizeGL(int, int);
+
       /**
        * Virtual function called when the GL area needs repainting.
        */
@@ -531,11 +518,6 @@ namespace Avogadro {
        */
       virtual void resizeEvent(QResizeEvent *event);
 
-      /**
-       * Virtual function called whn the GL area is resized
-       */
-      virtual void resizeGL(int, int);
-
       /**
        * Focus Event
        */
@@ -561,6 +543,18 @@ namespace Avogadro {
        */
       virtual void wheelEvent(QWheelEvent * event);
 
+      /**
+       * Response to key press events.
+       * @param event the key event information
+       */
+      virtual void keyPressEvent(QKeyEvent *event);
+
+      /**
+       * Response to key release events.
+       * @param event the key event information
+       */
+      virtual void keyReleaseEvent(QKeyEvent *event);
+
       /**
        * Render the scene. To be used in both modes GL_RENDER and GL_SELECT.
        * This function calls the render functions of the engines as well as the
@@ -594,11 +588,6 @@ namespace Avogadro {
        */
       virtual void renderDebugOverlay();
 
-      /**
-       * Helper function to load all engine factories.
-       */
-      void loadEngineFactories();
-
       /**
        * This will return a painting condition that must be met each time
        * before a GLThread can run.
@@ -621,6 +610,9 @@ namespace Avogadro {
        */
       inline double computeFramesPerSecond();
 
+      bool              m_glslEnabled;
+      Tool*             m_navigateTool; /// NavigateTool is a super tool
+
     public Q_SLOTS:
 
       /**
@@ -628,35 +620,6 @@ namespace Avogadro {
        */
       void setTool(Tool *tool);
 
-      /**
-       * Add the primitive to the widget.  This slot is called whenever
-       * a new primitive is added to our molecule model.  It adds the
-       * primitive to the list in the appropriate group.
-       *
-       * @param primitive pointer to a primitive to add to the view
-       */
-      void addPrimitive(Primitive *primitive);
-
-      /**
-       * Update a primitive.  This slot is called whenever a primitive of our
-       * molecule model has been changed and we need to check our view.
-       *
-       * @note In some cases we are passed the molecule itself meaning that more
-       * than one thing has changed in the molecule.
-       *
-       * @param primitive object which changed.
-       */
-      void updatePrimitive(Primitive *primitive);
-
-      /** Remove a primitive.  This slot is called whenever a primitive of our
-       * molecule model has been removed and we need to take it off our list.
-       * Additionally we need to update other items in our view that are impacted
-       * by this change.
-       *
-       * @param primitive object to remove.
-       */
-      void removePrimitive(Primitive *primitive);
-
       /**
        * Set the background color of the rendering area (the default is black).
        *
@@ -671,11 +634,28 @@ namespace Avogadro {
       void setMolecule(Molecule *molecule);
 
       /**
+       * A primitive, was removed, so update the selection
+       */
+      void unselectPrimitive(Primitive *);
+
+      /**
+       * An atom, was removed, so update the selection
+       */
+      void unselectAtom(Atom *);
+
+      /**
+       * A bond, was removed, so update the selection
+       */
+      void unselectBond(Bond *);
+
+      /**
+       * Add an engine to the GLWidget. 
        * @param engine Engine to add to this widget.
        */
       void addEngine(Engine *engine);
 
       /**
+       * Remove an engine from the GLWidget. 
        * @param engine Engine to remove from this widget.
        */
       void removeEngine(Engine *engine);
@@ -685,36 +665,46 @@ namespace Avogadro {
        */
       void loadDefaultEngines();
 
+      /**
+       * Reload the engine plugins.
+       */
+      void reloadEngines();
+
       /**
        * Signal that something changed and the display lists should be invalidated.
        */
       void invalidateDLs();
 
+      /**
+       * One or more tools are deleted..
+       */
+      void toolsDestroyed();
+
     Q_SIGNALS:
       /**
-       * Signal for the mouse press event which is passed to the engines and tools.
+       * Signal for the mouse press event which is passed to the tools.
        */
       void mousePress(QMouseEvent * event);
 
       /**
-       * Signal for the mouse release event which is passed to the engines and tools.
+       * Signal for the mouse release event which is passed to the tools.
        */
       void mouseRelease( QMouseEvent * event );
 
       /**
-       * Signal for the mouse move event which is passed to the engines and tools.
+       * Signal for the mouse move event which is passed to the tools.
        */
       void mouseMove( QMouseEvent * event );
 
       /**
-       * Signal for the mouse wheel event which is passed to the engines and tools.
+       * Signal for the mouse wheel event which is passed to the tools.
        */
       void wheel( QWheelEvent * event);
 
       /**
        * Signal that the Molecule has changed.
        */
-      void moleculeChanged(Molecule *previous, Molecule *next);
+      void moleculeChanged(Molecule *newMolecule);
 
       /**
        * Signal that an Engine has been added to the GLWidget.
@@ -726,6 +716,27 @@ namespace Avogadro {
        */
       void engineRemoved(Engine *engine);
 
+      /**
+       * Signal that the GLWidget has resized. Can be used to update child
+       * widgets, see overlay extension for example.
+       */
+      void resized();
+
+      /**
+       * Named selections have changed.
+       */
+      void namedSelectionsChanged();
+
+      /**
+       * Signal that this GLWidget was activated.
+       */
+      void activated(GLWidget *);
+
+      /**
+       * Signal that the unit cell axis display has changed
+       */
+      void unitCellAxesRenderChanged(bool enabled);
+
   };
 
 } // end namespace Avogadro
